/// ABBYY Mobile Capture © 2020 ABBYY Development, Inc.
/// ABBYY is a registered trademark or a trademark of ABBYY Software Ltd.

#import "AUICaptureScenario.h"
#import "AUIPageStorage.h"

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, AUIMultiPageResourceType) {
    /// Document name to display in camera.
    AUIMultiPageResourceTypeDocumentName,
    /// Page name to display in camera.
    AUIMultiPageResourceTypePageName,
    /// Capturing status to display as capturing state.
    AUIMultiPageResourceTypeCapturingStatus,
    /// Use this tip to advise user what document page place in front of the camera.
    AUIMultiPageResourceTypeLookingForPageTip,
    /// Text to display while scenario waits to manual capture.
    AUIMultiPageResourceTypeManualCaptureTip,
    /// Text to display in preview in not filled image slot, for example: Tap to add image of the third document page.
    AUIMultiPageResourceTypeAddPageTip,
};

/**
 The type of the Paging.
 
 The scenario can allow adding pages unlimitedly, can require specific count of pages and can require specific count of pages with unlimited count of optional pages.
 */
typedef NS_ENUM(NSUInteger, AUIMultiPageCaprureScenarioPagingType) {
    /// Use this Paging type if you want allow users to add unlimited count of pages.
    AUIMultiPageCaprureScenarioPagingTypeUnlimitedPages,
    /// Use this Paging type if you want require users to add specific count of pages.
    AUIMultiPageCaprureScenarioPagingTypeRequiredPages,
    /// Use this Paging type if you want require users to add specific count of pages with any count of optional pages.
    AUIMultiPageCaprureScenarioPagingTypeRequiredPagesWithUnlimitedOptionalPages
};

ABBYYUI_EXTERN
@interface AUIMultiPageCaptureScenarioPaging: NSObject

/// The type of the Paging.
@property (nonatomic, assign, readonly) AUIMultiPageCaprureScenarioPagingType type;
/// Pages required to add by the user.
///
/// For `UnlimitedPages` type the requiredPageCount must be 0.
///
/// For `RequiredPages` type the requiredPageCount must be greater than 0.
///
/// For `RequiredPagesWithUnlimitedOptionalPages` type the requiredPageCount must be greater than 0. User will have to add required pages first, after completing it the user will be able to add any amount of optional pages.
@property (nonatomic, assign, readonly) NSUInteger requiredPageCount;

+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;
- (instancetype)initWithType:(AUIMultiPageCaprureScenarioPagingType)type requiredPageCount:(NSUInteger)requiredPageCount;

@end

/**
 Additional page info datasource. All methods are optional.
 */
@protocol AUICustomResourceProvider <NSObject>

@optional
/**
 Returns text to show for end-user for specific page.

 @param captureScenario Capture scenario informing delegate.
 @param event Specific event.
 @param index image index.
 @return Returns text to show for end-user for specific page or nil to display default text.
 */
- (nullable NSString*)captureScenario:(AUICaptureScenario*)captureScenario
    stringForResourceType:(AUIMultiPageResourceType)event
    forPageAtIndex:(NSUInteger)index;

@end

ABBYYUI_EXTERN
@interface AUIMultiPageCaptureScenario : AUICaptureScenario

/// Paging behaviour for `AUIMultiPageCaptureScenario`.
///
/// Default is nil, meaning that the Paging is set to `UnlimitedPages` and its `requiredPageCount` is set to 0.
/// That is, the user will be able to add pages unlimitedly.
@property (nonatomic, strong, null_resettable) AUIMultiPageCaptureScenarioPaging* paging;
/// A custom resource provider implemented by the client of the scenario that is used for providing custom string resources like LookingForPage tip or some page's name.
@property (nonatomic, weak, nullable) id<AUICustomResourceProvider> customResourceProvider;
/// To show preview of the captured page. Default YES.
@property (nonatomic, assign) BOOL isShowPreviewEnabled;
/// The possibility to finish the scenario if not all required pages are added. Default NO.
@property (nonatomic, assign) BOOL canFinishUncompletedScenario;
/// Record id to show preview. You can specify record id to start with preview rather than camera. If nil scenario will be started with camera, otherwise - with preview of specified image.
@property (nonatomic, copy, nullable) AUIPageId startAsEditorAtPageId;

/// Subclass to use.
+ (instancetype)new __unavailable;
- (instancetype)init __unavailable;

@end

NS_ASSUME_NONNULL_END
