/// ABBYY® Mobile Capture © 2019 ABBYY Production LLC.
/// ABBYY is a registered trademark or a trademark of ABBYY Software Ltd.

#import <Foundation/Foundation.h>
#import "AUIPageAttributes.h"

/**
 ENUM describing which image type has to be saved to or loaded from the storage.
 */
typedef NS_ENUM(NSUInteger, AUIPageStorageImageType) {
    /// Original image.
    AUIPageStorageImageTypeOriginal,
    /// Cropped image.
    AUIPageStorageImageTypeCropped,
    /// Thumbnail image.
    AUIPageStorageImageTypeThumbnail
};

NS_ASSUME_NONNULL_BEGIN

typedef NSString* AUIPageId;

#pragma mark -

/**
 Storage API. You can implement custom storage if in some reason the default one (see 'AUIImageStorageFactory' interface) is not acceptable. For advanced users.
 */
@protocol AUIPageStorage <NSObject>

/**
 Asks to write attributes to the storage and expects created id for this record.
 
 @param attributes The attributes that should be saved while creating the image.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns identifier associated with new record in the storage.
 */
- (nullable AUIPageId)createPageWithAttributes:(AUIPageAttributes*)attributes withError:(NSError**)error;

/**
 Returns the attributes associated with a given pageId.
 
 @param identifier Record id.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns the data associated with a given identifier or nil on error.
 */
- (nullable AUIPageAttributes*)attributesForPage:(AUIPageId)identifier error:(NSError**)error;

/**
 Returns the image associated with a given record id and type.
 
 @param type Image type. Can be Original, Cropped or Thumbnail.
 @param identifier Record id.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns UIImage on success or nil on error.
 */
- (nullable UIImage*)imageOfType:(AUIPageStorageImageType)type forPage:(AUIPageId)identifier error:(NSError**)error;

/**
 Asks to update page's attributes for concrete record.
 
 @param attributes Pages attributes that need to be updated.
 @param identifier Record id.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns YES on success or NO on error.
 */
- (BOOL)updatePageAttributes:(AUIPageAttributes*)attributes forPage:(AUIPageId)identifier error:(NSError**)error;

/**
 Asks to store image of specified type for concrete record id.
 
 @param image The UIImage.
 @param type Image type. Can be Original, Cropped or Thumbnail.
 @param identifier Record id.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns YES on success or NO on error.
 */
- (BOOL)storeImage:(nullable UIImage*)image ofType:(AUIPageStorageImageType)type forPage:(AUIPageId)identifier error:(NSError**)error;

/**
 Removes record from storage.

 @param identifier Record id.
 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns YES on success or NO on error.
 */
- (BOOL)deletePage:(AUIPageId)identifier error:(NSError**)error;

/**
 Returns all record identifers in the storage.

 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns all record identifers or nil on error.
 */
- (nullable NSArray<AUIPageId>*)pagesWithError:(NSError**)error;

/**
 Removes all images.

 @param error If there is an error, upon return contains an error object that describes the problem.
 @return Returns YES on success or NO on error.
 */
- (BOOL)clearWithError:(NSError**)error;

@end

NS_ASSUME_NONNULL_END
