// Copyright (C) © 2024 ABBYY Development, Inc., 2024. All rights reserved.
// Product: ABBYY® VantageMobileInputSDKSampleSwift®
// JSONInputViewController.swift
// Description: Input JSON screen

import UIKit

/// JSON input screen completion.
typealias JSONInputCompletion = (_ jsonData: Data?) -> Void

class JSONInputViewController: UIViewController {
    
    /// Completion.
    var completion: JSONInputCompletion?
    
    /// Placeholder label.
    @IBOutlet private var placeholderLabel: UILabel!
    /// JSON input text view.
    @IBOutlet private var textView: UITextView!
    /// Capture button.
    @IBOutlet private var captureButton: UIButton!
    /// Keyboard constraint.
    @IBOutlet private var keyboardConstraint: NSLayoutConstraint!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        navigationItem.leftBarButtonItem = .init(image: UIImage(systemName: "xmark"), style: .plain, target: self, action: #selector(closeButtonPressed(_:)))
        
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardLayoutUpdate), name: UIResponder.keyboardWillShowNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardLayoutUpdate), name: UIResponder.keyboardWillChangeFrameNotification, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(keyboardLayoutUpdate), name: UIResponder.keyboardDidChangeFrameNotification, object: nil)
        
        textView.delegate = self
        enableCaptureButton(false)
        
        configureUI()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        textView.becomeFirstResponder()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        view.endEditing(true)
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        NotificationCenter.default.removeObserver(self)
    }
    
    // MARK: - Actions
    
    /// The capture button is pressed. Convert JSON text from text view to Data.
    @IBAction private func captureButtonPressed(_ sender: Any) {
        let jsonString = textView.text!.filter { !$0.isWhitespace || ($0 == " ") }
        guard let jsonData = jsonString.data(using: .utf8) else {
            return
        }
        finishInput(jsonData: jsonData)
    }
    
    // MARK: - Private Methods
    
    /// Configure UI.
    private func configureUI() {
        textView.layer.cornerRadius = 8.0
        textView.layer.borderWidth = 0.5
        textView.layer.borderColor = UIColor(red: 0.78, green: 0.78, blue: 0.78, alpha: 1.00).cgColor
        textView.textContainerInset = .init(top: 11.0, left: 13.0, bottom: 11.0, right: 13.0)
        
        captureButton.layer.cornerRadius = 14.0
        captureButton.layer.borderWidth = 1.5
    }
    
    /// Switching Capture button modes.
    private func enableCaptureButton(_ enable: Bool) {
        captureButton.isEnabled = enable
        captureButton.backgroundColor = enable ? UIColor(named: "main_color") : UIColor(red: 0.93, green: 0.93, blue: 0.93, alpha: 1.00)
        captureButton.layer.borderColor = enable ? UIColor(named: "main_color")!.cgColor : UIColor(red: 0.82, green: 0.83, blue: 0.85, alpha: 1.00).cgColor
        captureButton.tintColor = enable ? .white : UIColor(red: 0.67, green: 0.69, blue: 0.74, alpha: 1.00)
    }
    
    /// Close controller.
    @objc private func closeButtonPressed(_ sender: Any) {
        finishInput(jsonData: nil)
    }
    
    /// Update items when the keyboard is displaying or resized.
    @objc private func keyboardLayoutUpdate(_ notification: Notification) {
        if let keyboardFrame: NSValue = notification.userInfo?[UIResponder.keyboardFrameEndUserInfoKey] as? NSValue,
            let duration: NSNumber = notification.userInfo?[UIResponder.keyboardAnimationDurationUserInfoKey] as? NSNumber {
            let keyboardHeight = keyboardFrame.cgRectValue.height
            keyboardConstraint.constant = keyboardHeight + 16.0
            UIView.animate(withDuration: duration.doubleValue) {
                self.view.layoutIfNeeded()
            }
        }
    }
    
    private func finishInput(jsonData: Data?) {
        dismiss(animated: true) {
            self.completion?(jsonData)
        }
    }
}

/// Implementation of ``UITextViewDelegate``.
extension JSONInputViewController: UITextViewDelegate {
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        let newText = textView.text.replacingCharacters(in: Range(range, in: textView.text)!, with: text)
        placeholderLabel.isHidden = !newText.isEmpty
        enableCaptureButton(!newText.isEmpty)
        
        return true
    }
}
