// Copyright (C) © 2024 ABBYY Development, Inc., 2024. All rights reserved.
// Product: ABBYY® VantageMobileInputSDKSampleSwift®
// QRCodeScanViewController.swift
// Description: Scan QR code camera screen.

import UIKit
import AVFoundation

/// QR code camera screen completion
typealias QRCodeScanCompletion = (_ urlString: String?) -> Void

/// QR code camera screen
class QRCodeScanViewController: UIViewController {
    
    /// Error view.
    @IBOutlet private var cameraErrorView: UIView!
    /// Error label.
    @IBOutlet private var errorLabel: UILabel!
    
    /// Completion.
    var completion: QRCodeScanCompletion?
    /// Capture session.
    private var captureSession: AVCaptureSession!
    /// Video preview layer.
    private var videoPreviewLayer: AVCaptureVideoPreviewLayer? {
        didSet {
            videoPreviewLayer?.removeFromSuperlayer()
            if let layer = videoPreviewLayer {
                layer.frame = self.view.layer.bounds
                self.view.layer.insertSublayer(layer, at: 0)
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        requestCameraAccess()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        if let connection = videoPreviewLayer?.connection {
            let currentDevice = UIDevice.current
            let orientation: UIDeviceOrientation = currentDevice.orientation
            let previewLayerConnection: AVCaptureConnection = connection
            
            if previewLayerConnection.isVideoOrientationSupported {
                var layerOrientation: AVCaptureVideoOrientation = .portrait
                switch orientation {
                    case .portrait:
                        layerOrientation = .portrait
                    case .landscapeRight:
                        layerOrientation = .landscapeLeft
                    case .landscapeLeft:
                        layerOrientation = .landscapeRight
                    case .portraitUpsideDown:
                        layerOrientation = .portraitUpsideDown
                    default:
                        break
                }
                previewLayerConnection.videoOrientation = layerOrientation
                videoPreviewLayer?.frame = view.bounds
            }
        }
    }
    
    // MARK: - Actions
    
    /// Close controller.
    @IBAction private func closeButtonPressed(_ sender: Any) {
        finishScan(nil)
    }
    
    /// Open system settings.
    @IBAction private func settingsButtonPressed(_ sender: Any) {
        guard let settingsUrl = URL(string: UIApplication.openSettingsURLString) else {
            return
        }
        
        if UIApplication.shared.canOpenURL(settingsUrl) {
            UIApplication.shared.open(settingsUrl)
        }
    }
    
    // MARK: - Private Methods
    
    /// Show error.
    private func showError(_ errorString: String?) {
        errorLabel.text = errorString
        cameraErrorView.isHidden = errorString == nil
    }
    
    /// Request camera access.
    private func requestCameraAccess() {
        switch AVCaptureDevice.authorizationStatus(for: .video) {
            case .notDetermined:
                AVCaptureDevice.requestAccess(for: .video) { accessGranted in
                    DispatchQueue.main.async {
                        guard accessGranted else {
                            self.showError("No camera access")
                            return
                        }
                        self.setupCaptureSession()
                    }
                }
            case .denied, .restricted:
                showError("No camera access")
            case .authorized:
                setupCaptureSession()
            @unknown default:
                fatalError()
        }
    }
    
    /// Configure capture session.
    private func setupCaptureSession() {
        do {
            guard let captureDevice = AVCaptureDevice.default(.builtInWideAngleCamera, for: .video, position: .back) else {
                showError("Unsupported device")
                return
            }
            
            // Get an instance of the AVCaptureDeviceInput class using the previous device object.
            let input = try AVCaptureDeviceInput(device: captureDevice)
            
            // Initialize AVCaptureSession
            let captureSession = AVCaptureSession()
            self.captureSession = captureSession
            
            // Set the input device on the capture session.
            captureSession.addInput(input)
            
            // Initialize a AVCaptureMetadataOutput object and set it as the output device to the capture session.
            let captureMetadataOutput = AVCaptureMetadataOutput()
            captureSession.addOutput(captureMetadataOutput)
            
            // Set delegate and use the default dispatch queue to execute the call back
            captureMetadataOutput.setMetadataObjectsDelegate(self, queue: DispatchQueue.main)
            captureMetadataOutput.metadataObjectTypes = [AVMetadataObject.ObjectType.qr]
            
            // Initialize the video preview layer and add it as a sublayer to the viewPreview view's layer.
            videoPreviewLayer = AVCaptureVideoPreviewLayer(session: captureSession)
            videoPreviewLayer?.videoGravity = AVLayerVideoGravity.resizeAspectFill
            
            DispatchQueue.global(qos: .userInitiated).async { [weak self] in
                self?.captureSession.startRunning()
            }
        } catch {
            showError(error.localizedDescription)
        }
    }
    
    private func finishScan(_ urlString: String?) {
        dismiss(animated: true) {
            self.completion?(urlString)
            self.completion = nil
        }
    }
}

/// Implementation of ``AVCaptureMetadataOutputObjectsDelegate``.
extension QRCodeScanViewController: AVCaptureMetadataOutputObjectsDelegate {
    func metadataOutput(_ output: AVCaptureMetadataOutput, didOutput metadataObjects: [AVMetadataObject], from connection: AVCaptureConnection) {
        // Check if the metadataObjects array contains at least one object. Get the metadata object.
        guard let metadataObj = metadataObjects.first as? AVMetadataMachineReadableCodeObject else {
            return
        }
        
        guard metadataObj.type == AVMetadataObject.ObjectType.qr else {
            return
        }
        
        if let string = metadataObj.stringValue {
            finishScan(string)
        }
    }
}
