// Copyright (C) © 2024 ABBYY Development, Inc., 2024. All rights reserved.
// Product: ABBYY® VantageMobileInputSDKSampleSwift®
// SessionCell.swift
// Description: Session table view cell.

import UIKit
import AVFoundation
import VantageMobileInput

/// Session cell delegate. Notified about selected session action.
protocol SessionCellDelegate: AnyObject {
    func sessionCell(_ cell: SessionCell, didSelectAction action: SessionCellAction)
}

/// Session action that can be applied for session.
enum SessionCellAction {
    /// Upload documents for session with specified identifier.
    case upload
    /// Delete session with specified identifier.
    case delete
}

/// Session cell.
class SessionCell: UITableViewCell {
    private enum Constants {
        static let imageViewSize = CGSize(width: 84.0, height: 84.0)
        static let stubImageViewSize = CGSize(width: 46.0, height: 31.0)
    }
    
    @IBOutlet private weak var thumbnailImageView: UIImageView!
    @IBOutlet private weak var thumbnailImageViewHeight: NSLayoutConstraint!
    @IBOutlet private weak var thumbnailImageViewWidth: NSLayoutConstraint!
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var numberPagesLabel: UILabel!
    @IBOutlet private weak var dateCreationLabel: UILabel!
    @IBOutlet private weak var optionsButton: UIButton!
    @IBOutlet private weak var activityIndicator: UIActivityIndicatorView!
    
    weak var delegate: SessionCellDelegate?
    
    var config: SessionCellConfig! {
        didSet {
            setupPreviewImageView(from: config.info.thumbnail, orientation: config.info.thumbnailOrentation)
            titleLabel.text = String(format: String(localized: "%lld documents"), config.info.capturedDocumentsCount)
            numberPagesLabel.text = String(format: String(localized: "%lld pages"), config.info.capturedPagesCount)
            dateCreationLabel.text = "Created \(SessionCell.dateFormatter.string(from: config.info.createdAt))"
            showActivityIndicator(config.isUploading)
            optionsButton.isHidden = config.isUploading
        }
    }
    
    private static let dateFormatter: DateFormatter = {
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "MM/dd/yyyy HH:mm"
        return dateFormatter
    }()
    
    override func awakeFromNib() {
        super.awakeFromNib()
        optionsButton.setTitle(nil, for: .normal)
        var menuActions: [UIMenuElement] = []
        menuActions.append(UIAction(title: "Upload", image: UIImage(systemName: "square.and.arrow.up"), handler: { [weak self] _ in
            if let self {
                self.notifyAboutAction(.upload)
            }
        }))
        menuActions.append(UIAction(title: "Delete", image: UIImage(systemName: "trash"), handler: { [weak self] _ in
            if let self {
                self.notifyAboutAction(.delete)
            }
        }))
        optionsButton.menu = UIMenu(children: menuActions)
        optionsButton.showsMenuAsPrimaryAction = true
        
        selectedBackgroundView = UIView()
        selectedBackgroundView?.backgroundColor = UIColor.black.withAlphaComponent(0.06)
    }
    
    private func setupPreviewImageView(from imageURL: URL?, orientation: UIImage.Orientation?) {
        var image: UIImage? = nil
        if let path = imageURL?.path, let uiImage = UIImage(contentsOfFile: path) {
            if let cgImage = uiImage.cgImage {
                image = UIImage(cgImage: cgImage, scale: 1, orientation: orientation!)
            } else if let ciImage = uiImage.ciImage {
                image = UIImage(ciImage: ciImage, scale: 1, orientation: orientation!)
            }
        }
        
        if image != nil {
            thumbnailImageView.layer.shadowRadius = 1.5
            thumbnailImageView.layer.shadowOpacity = 0.25
            thumbnailImageView.layer.shadowColor = UIColor.black.cgColor
            thumbnailImageView.layer.shadowOffset = .init(width: 0, height: 1)
        } else {
            thumbnailImageView.layer.shadowRadius = .zero
            thumbnailImageView.layer.shadowOpacity = .zero
            thumbnailImageView.layer.shadowColor = nil
            thumbnailImageView.layer.shadowOffset = .zero
        }
        
        let config = configForPreviewImage(for: image)
        thumbnailImageView.image = config.image
        thumbnailImageViewHeight.constant = config.size.height
        thumbnailImageViewWidth.constant = config.size.width
        thumbnailImageView.layoutIfNeeded()
    }
    
    private func configForPreviewImage(for image: UIImage?) -> (image: UIImage, size: CGSize) {
        guard let image else {
            // Handle the case of a stub image
            return (UIImage(systemName: "photo")!, Constants.stubImageViewSize)
        }
        
        return (image, Constants.imageViewSize)
    }
    
    private func showActivityIndicator(_ show: Bool) {
        activityIndicator.isHidden = !show
        show ? activityIndicator.startAnimating() : activityIndicator.stopAnimating()
    }
    
    private func notifyAboutAction(_ action: SessionCellAction) {
        delegate?.sessionCell(self, didSelectAction: action)
    }
}
