// Copyright (C) © 2024 ABBYY Development, Inc., 2024. All rights reserved.
// Product: ABBYY® VantageMobileInputSDKSampleSwift®
// SessionsViewController.swift
// Description: Main screen that displays list of active capturing sessions.

import UIKit
import VantageMobileInput

/// Session mode.
enum SessionMode {
    /// New session.
    case new
    /// Continue session with specified identifier.
    case `continue`(sessionIdentifier: String)
    /// Upload documents for session with specified identifier.
    case upload(sessionIdentifier: String)
    
    /// Completion action button title for parameters screen.
    var actionButtonTitle: String {
        switch self {
            case .new:
                return "Start Capture"
            case .continue:
                return "Continue Capture"
            case .upload:
                return "Start Uploading"
        }
    }
    
    /// Session identifier for current session mode. Returns nil for new session mode.
    var sessionIdentifier: String? {
        switch self {
            case .new:
                return nil
            case let .continue(sessionIdentifier), let .upload(sessionIdentifier):
                return sessionIdentifier
        }
    }
}

/// Constants.
private enum Constants {
    /// License file name.
    static let licenseFileName = "license"
    /// Key to store mobile input parameters in UserDefaults.
    static let mobileInputParametersKey = "MobileInputParameters"
    /// Key to store documents to scan  in UserDefaults.
    static let documentsToScanKey = "DocumentsToScan"
}

/// Session cell configuration.
struct SessionCellConfig {
    let info: MobileInputDocuments.SessionInfo
    var isUploading: Bool
}

/// Documents to scan type
enum DocumentsToScanType: String, CaseIterable {
    /// JSON's file url that describes document.
    case webURL = "Mobile Upload Link"
    /// Multiple documents.
    case multiple = "Multiple Documents"
    /// Single document.
    case single = "Single Document"
    /// ID card.
    case id = "ID Card"
    /// Passport B6.
    case passport = "Passport"
    /// JSON's that describes document.
    case json = "JSON Document Schema"
}

class SessionsViewController: UIViewController, UITableViewDataSource, UITableViewDelegate, SessionCellDelegate {
    
    /// Table view for active sesssions.
    @IBOutlet private weak var tableView: UITableView!
    /// Stub empty transactions.
    @IBOutlet private weak var stubStackView: UIStackView!
    /// New session button.
    @IBOutlet private weak var newSessionButton: UIButton!
    /// Action sheet alert controller.
    private weak var actionSheetAlertController: UIAlertController?
    
    /// Session mode.
    private var sessionMode: SessionMode = .new
    /// Active sessions.
    private var sessions: [SessionCellConfig] = []
    
    /// Mobile input parameters for saved sessions.
    private var mobileInputParameters: [String: MobileInputDocuments.MobileInputParameters] = [:]
    /// Documents to scan for sessions saved in disconnected mode.
    private var documentsToScan: [String: MobileInputDocuments.MobileInputParameters.DocumentsToScan] = [:]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        load()
        configure()
        reload()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        reload()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        if let actionSheetAlertController {
            actionSheetAlertController.popoverPresentationController?.sourceRect = newSessionButton.frame
        }
    }
    
    // MARK: - Actions
    
    @IBAction private func newSessionButtonPressed(_ sender: Any) {
        startNewSession()
    }
    
    // MARK: - Private Methods
    
    /// Load saved parameters for sessions continuation.
    private func load() {
        if let data = UserDefaults.standard.data(forKey: Constants.mobileInputParametersKey), let mobileInputParameters = try? JSONDecoder().decode([String: MobileInputDocuments.MobileInputParameters].self, from: data) {
            self.mobileInputParameters = mobileInputParameters
        }
        if let data = UserDefaults.standard.data(forKey: Constants.documentsToScanKey), let documentsToScan = try? JSONDecoder().decode([String: MobileInputDocuments.MobileInputParameters.DocumentsToScan].self, from: data) {
            self.documentsToScan = documentsToScan
        }
    }
    
    /// Save parameters for sessions continuation.
    private func save() {
        if let data = try? JSONEncoder().encode(mobileInputParameters) {
            UserDefaults.standard.setValue(data, forKey: Constants.mobileInputParametersKey)
        }
        if let data = try? JSONEncoder().encode(documentsToScan) {
            UserDefaults.standard.setValue(data, forKey: Constants.documentsToScanKey)
        }
    }
    
    /// Configure UI.
    private func configure() {
        newSessionButton.isHidden = false
        newSessionButton.layer.masksToBounds = false
        newSessionButton.layer.cornerRadius = 14.0
        newSessionButton.layer.shadowColor = UIColor(named: "main_color")!.cgColor
        newSessionButton.layer.shadowPath = UIBezierPath(roundedRect: newSessionButton.bounds, cornerRadius: newSessionButton.layer.cornerRadius).cgPath
        newSessionButton.layer.shadowOffset = CGSize(width: 0.0, height: 8.0)
        newSessionButton.layer.shadowOpacity = 0.5
        newSessionButton.layer.shadowRadius = 10.0
        
        tableView.separatorStyle = .none
        tableView.contentInset = .init(top: .zero, left: .zero, bottom: newSessionButton.bounds.height + 20.0, right: .zero)
    }
    
    /// Reload sessions and display.
    private func reload() {
        sessions = MobileInputDocuments.shared.savedSessions.map { sessionInfo in
            if let oldSession = sessions.first(where: { $0.info.sessionIdentifier == sessionInfo.sessionIdentifier }) {
                return SessionCellConfig(info: sessionInfo, isUploading: oldSession.isUploading)
            } else {
                return SessionCellConfig(info: sessionInfo, isUploading: false)
            }
        }
        tableView.reloadData()
        showStub(sessions.isEmpty)
        
        let savedSessions = Set(sessions.map { $0.info.sessionIdentifier })
        // Remove saved mobile input parameters for deleted sessions.
        var deletedSessions = savedSessions.subtracting(Set(mobileInputParameters.keys))
        for session in deletedSessions {
            mobileInputParameters.removeValue(forKey: session)
        }
        
        // Remove saved documents to scan for deleted sessions.
        deletedSessions = savedSessions.subtracting(Set(documentsToScan.keys))
        for session in deletedSessions {
            documentsToScan.removeValue(forKey: session)
        }
        save()
    }
    
    /// Show stub.
    private func showStub(_ show: Bool) {
        stubStackView.isHidden = !show
    }
    
    /// Start a new session.
    private func startNewSession() {
        showDocumentsToScanPicker { [weak self] scanMode in
            guard let self else {
                return
            }
            let sessionIdentifier = MobileInputDocuments.shared.scanDocuments(scanMode: scanMode, presenter: self) { [weak self] result in
                self?.handleResult(result, sessionIdentifier: nil, scanMode: scanMode)
                self?.reload()
            }
            switch scanMode {
                case let .scanAndUpload(mobileInputParameters):
                    self.mobileInputParameters[sessionIdentifier] = mobileInputParameters
                    self.documentsToScan[sessionIdentifier] = mobileInputParameters.documentsToScan
                case let .scan(documents):
                    self.documentsToScan[sessionIdentifier] = documents
            }
            self.save()
        }
    }
    
    /// Continue session with specified info.
    private func continueSession(_ sessionInfo: MobileInputDocuments.SessionInfo) {
        let scanMode: MobileInputDocuments.ScanMode
        if let mobileInputParameters = mobileInputParameters[sessionInfo.sessionIdentifier] {
            scanMode = .scanAndUpload(mobileInputParameters: mobileInputParameters)
        } else {
            scanMode = .scan(documents: documentsToScan[sessionInfo.sessionIdentifier]!)
        }
        
        MobileInputDocuments.shared.continueScanningDocuments(scanMode: scanMode,
                                                              sessionIdentifier: sessionInfo.sessionIdentifier,
                                                              presenter: self)
        { [weak self] result in
            self?.handleResult(result,
                               sessionIdentifier: sessionInfo.sessionIdentifier,
                               scanMode: scanMode,
                               onRetry: {
                                   // Expired parameters. Ask for new one
                                   self?.showMobileUploadLinkInputAlert(completion: { parameters in
                                       self?.mobileInputParameters[sessionInfo.sessionIdentifier] = parameters
                                       self?.documentsToScan[sessionInfo.sessionIdentifier] = parameters.documentsToScan
                                       self?.continueSession(sessionInfo)
                                   })
                               })
            self?.reload()
        }
    }
    
    /// Upload session with specified info.
    private func upload(_ sessionInfo: MobileInputDocuments.SessionInfo) {
        let uploadWithParameters: (_ parameters: MobileInputDocuments.MobileInputParameters) -> Void = { [weak self] parameters in
            guard let self else {
                return
            }
            let index = sessions.firstIndex(where: { $0.info.sessionIdentifier == sessionInfo.sessionIdentifier })!
            sessions[index].isUploading = true
            reload()
            let alert = UIAlertController(title: "Uploading...", message: nil, preferredStyle: .alert)
            present(alert, animated: true)
            
            MobileInputDocuments.shared.uploadDocuments(mobileInputParameters: parameters,
                                                        sessionIdentifier: sessionInfo.sessionIdentifier,
                                                        progressHandler: nil)
            { [weak self] result in
                self?.sessions[index].isUploading = false
                alert.dismiss(animated: true) {
                    self?.handleResult(result,
                                       sessionIdentifier: sessionInfo.sessionIdentifier,
                                       scanMode: .scanAndUpload(mobileInputParameters: parameters),
                                       onRetry: {
                                           // Expired parameters. Ask for new one
                                           self?.showMobileUploadLinkInputAlert(completion: { parameters in
                                               self?.mobileInputParameters[sessionInfo.sessionIdentifier] = parameters
                                               self?.documentsToScan[sessionInfo.sessionIdentifier] = parameters.documentsToScan
                                               self?.upload(sessionInfo)
                                           })
                                       })
                    self?.reload()
                }
            }
        }
        
        if let parameters = mobileInputParameters[sessionInfo.sessionIdentifier] {
            uploadWithParameters(parameters)
        } else {
            showMobileUploadLinkInputAlert { parameteres in
                uploadWithParameters(parameteres)
            }
        }
    }
    
    /// Delete session with specified info.
    private func deleteSession(_ sessionIdentifier: MobileInputDocuments.SessionIdentifier) {
        if let error = MobileInputDocuments.shared.deleteSession(sessionIdentifier) {
            showErrorAlert(error: error)
        }
        reload()
    }
    
    /// Show an action sheet to select the DocumentsToScan type.
    private func showDocumentsToScanPicker(completion: @escaping (_ scanMode: MobileInputDocuments.ScanMode) -> Void) {
        let actionSheet = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
        actionSheet.addAction(UIAlertAction(title: "Cancel", style: .cancel))
        for type in DocumentsToScanType.allCases {
            let action = UIAlertAction(title: type.rawValue, style: .default, handler: { [weak self] _ in
                switch type {
                    case .webURL:
                        self?.showMobileUploadLinkInputAlert { parameters in
                            completion(.scanAndUpload(mobileInputParameters: parameters))
                        }
                    case .multiple:
                        completion(.scan(documents: MobileInputDocuments.MobileInputParameters.DocumentsToScan.multipleDocuments))
                    case .single:
                        completion(.scan(documents: MobileInputDocuments.MobileInputParameters.DocumentsToScan.singleDocument))
                    case .id:
                        completion(.scan(documents: MobileInputDocuments.MobileInputParameters.DocumentsToScan.idCard))
                    case .passport:
                        completion(.scan(documents: MobileInputDocuments.MobileInputParameters.DocumentsToScan.passport))
                    case .json:
                        self?.inputJSON { jsonData in
                            if let jsonData {
                                completion(.scan(documents: MobileInputDocuments.MobileInputParameters.DocumentsToScan.jsonData(jsonData)))
                            }
                        }
                }
            })
            actionSheet.addAction(action)
        }
        if UIDevice.current.userInterfaceIdiom == .pad {
            actionSheet.popoverPresentationController?.sourceView = view
            let sourceRect = newSessionButton.frame(forAlignmentRect: newSessionButton.frame)
            actionSheet.popoverPresentationController?.sourceRect = sourceRect
            actionSheetAlertController = actionSheet
        }
        present(actionSheet, animated: true)
    }
    
    /// Show an alert to enter Mobile Upload Link.
    private func showMobileUploadLinkInputAlert(presetURL: String? = nil, completion: @escaping (MobileInputDocuments.MobileInputParameters) -> Void) {
        let alert = UIAlertController(title: "Enter Mobile Upload link",
                                      message: "Scan QR code or enter Mobile Upload link",
                                      preferredStyle: .alert)
        alert.addTextField { textField in
            textField.placeholder = "https://"
            textField.text = presetURL
        }
        
        let enterAction = UIAlertAction(title: "Enter", style: .default, handler: { [weak self] _ in
            if let urlString = alert.textFields?.first?.text,
               let parameters = self?.parseURL(urlString)
            {
                completion(parameters)
            }
        })
        enterAction.isEnabled = !(presetURL ?? "").isEmpty
        alert.addAction(enterAction)
        alert.preferredAction = enterAction
        NotificationCenter.default.addObserver(forName: UITextField.textDidChangeNotification, object: alert.textFields?.first, queue: .main) { _ in
            enterAction.isEnabled = alert.textFields?.first?.text?.isEmpty == false
        }
        
        alert.addAction(UIAlertAction(title: "Scan QR code", style: .default, handler: { [weak self] _ in
            self?.scanQR { [weak self] urlString in
                if let urlString {
                    self?.showMobileUploadLinkInputAlert(presetURL: urlString, completion: completion)
                } else {
                    self?.showMobileUploadLinkInputAlert(presetURL: alert.textFields!.first!.text, completion: completion)
                }
            }
        }))
        alert.addAction(UIAlertAction(title: "Cancel", style: .default))
        present(alert, animated: true)
    }
    
    /// Open scan QR code screen.
    private func scanQR(_ completion: @escaping QRCodeScanCompletion) {
        let controller: QRCodeScanViewController = storyboard?.instantiateViewController(withIdentifier: "QRCodeScanViewController") as! QRCodeScanViewController
        controller.completion = completion
        present(controller, animated: true)
    }
    
    /// Open input mobile input JSON screen.
    private func inputJSON(_ completion: @escaping JSONInputCompletion) {
        let nc = storyboard?.instantiateViewController(withIdentifier: "JSONInputNavigationController") as! UINavigationController
        let controller = nc.viewControllers.first! as! JSONInputViewController
        controller.completion = completion
        present(nc, animated: true)
    }
    
    /// Parse URL to mobile input parameters.
    private func parseURL(_ urlString: String) -> MobileInputDocuments.MobileInputParameters? {
        guard let url = URL(string: urlString) else {
            showAlert(title: "Invalid URL")
            return nil
        }
        do {
            let mobileInputParameters = try MobileInputDocuments.shared.createMobileInputParameters(fromMobileInputLink: url)
            return mobileInputParameters
        } catch {
            let nsError = error as NSError
            
            let title = "Incorrect Mobile Input Link"
            let message = """
            Code: \(nsError.code)
            Failure reason: \(nsError.localizedFailureReason ?? "")
            """
            showAlert(title: title, message: message)
            return nil
        }
    }
    
    /// Handle session result.
    private func handleResult(_ result: Result<MobileInputDocuments.SessionIdentifier, NSError>,
                              sessionIdentifier: MobileInputDocuments.SessionIdentifier?,
                              scanMode: MobileInputDocuments.ScanMode,
                              onRetry: (() -> Void)? = nil)
    {
        switch result {
            case let .success(sessionIdentifier):
                // mobileInputParameters are saved into a dictionary when you're using the 'scanAndUpload' mode or uploading the documents
                if mobileInputParameters[sessionIdentifier] != nil {
                    showAlert(title: "Upload Successful")
                }
                // Delete the session only if it is successfully uploaded
                if case .scanAndUpload(mobileInputParameters: _) = scanMode {
                    deleteSession(sessionIdentifier)
                }
            case let .failure(error):
                if error.code == 1100 {
                    /// Scenario closed by user
                    return
                }
                showErrorAlert(error: error, onRetry: onRetry)
        }
    }
    
    func showErrorAlert(error: NSError, onRetry: (() -> Void)? = nil) {
        let title = error.localizedDescription
        let recoverySuggestion = error.localizedRecoverySuggestion ?? ""
        let details = """
        Code: \(error.code)
        Failure reason: \(error.localizedFailureReason ?? "")
        """
        showResultAlert(title: title, message: recoverySuggestion, details: details) {
            onRetry?()
        }
    }
    
    /// Show alert in main thread.
    private func showResultAlert(title: String, message: String?, details: String? = nil, completion: (() -> Void)? = nil) {
        if let message {
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.5) {
                self.showAlert(title: title, message: message, details: details, completion: completion)
            }
        }
    }
    
    // MARK: - UITableViewDataSource, UITableViewDelegate
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return sessions.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "SessionCell") as! SessionCell
        cell.config = sessions[indexPath.row]
        cell.delegate = self
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        tableView.deselectRow(at: indexPath, animated: true)
        let session = sessions[indexPath.row].info
        continueSession(session)
    }
    
    // MARK: - SessionCellDelegate
    
    func sessionCell(_ cell: SessionCell, didSelectAction action: SessionCellAction) {
        let sessionInfo = cell.config.info
        switch action {
            case .upload:
                upload(sessionInfo)
            case .delete:
                deleteSession(sessionInfo.sessionIdentifier)
        }
    }
}
